(function($){

	const DATA_OBJ_ID = 'idc_slider_obj';

	$.fn.idcSlider = function(action, params) 
	{
		//alert(action);
		switch (action) {
			
			case 'init':
				init(this, params);
				break;
				
			case 'next':
				next(this);
				break;
				
			case 'prev':
				prev(this);
				break;
			
			case 'animated':
				return animated(this);
				break;
		}
	}
	
	function animated(self)
	{
		var sObj = $(self).data(DATA_OBJ_ID);
		if (! sObj)
			return false;
		else
			return sObj.animated;
	}
	
	
	function init(self, params) 
	{
		$(self).css('display', 'none');
		$(self).css('opacity', '0');
		
		var sObj = new SliderObject(self, params);
		$(self).data(DATA_OBJ_ID, sObj);
		
		createHtml(self, sObj);
		var count = sObj.arr.length;

		//вешаем обработчики события загрузки рисунков
		var obj = {'cLoaded':0, 'cAll':count, 'self':self, 'did':DATA_OBJ_ID};
		obj.timeoutID = setTimeout(sObj.onLoadingError, 30000);
		$('img', self).bind('load', obj, function(event){
			
			
			event.data.cLoaded++;//плюсуем количество подгруженных рисунков
			//alert(event.data.cLoaded);
			//если подгрузились все рисунки
			if (event.data.cLoaded == event.data.cAll ) {
				
				//отвязываем эти обработчики загрузки картинок
				$('img', event.data.self).unbind('load');
				
				//вызываем пользовательский обработчик окончания инициализации
				var sObj = $(self).data(event.data.did);
				$(self).css('display', 'block');
				$(self).animate({'opacity':'1'}, 500);
				clearTimeout(event.data.timeoutID);
				event.data.timeoutID = null;
				
				if (sObj.onInited)
					sObj.onInited.call();
				
			}
		});


		for (var i=0; i < count; i++) {
			setImage(self, i, i);
		}
	}
	
	
	function setImage(self, n, m)
	{
		var img = $('img', self).eq(n);
		var sObj = $(self).data(DATA_OBJ_ID);
		img.attr('src', sObj.list[m]);
		//alert(sObj.list[m]);
	}
	
	function createHtml(self, sObj)
	{
		var html = '';
		
		var n = sObj.arr.length;
		var left = sObj.baseOffset;
		
		for (var i=0; i<n; i++) {
			html += '<img id="sl_img'+i+'" width="'+sObj.imgW+'" height="'+sObj.imgH+'" src=""/ style="position:absolute; left:'+left+'px">';
			left = left + sObj.dOffset;
		}
		
		$(self).html(html);
		//$('img', self).css('margin-left',sObj.margin+'px');
		$('img', self).css('float','left');
		
		
		$(self).css('width', sObj.scrW+'px');
		$(self).css('height', sObj.imgH);
		$(self).css('overflow', 'hidden');
		
		//alert($(self).html())
		//$(self).css('margin-left', sObj.baseOffset);
	}
	
	
	
	function next(self)
	{
		var sObj = $(self).data(DATA_OBJ_ID);
		if (sObj.current < sObj.arr.length - 1) {		

			animNext(sObj);
		}

	}
	
	function prev(self)
	{
		var sObj = $(self).data(DATA_OBJ_ID);
		if (sObj.current > 0) {
		   
		   animPrev(sObj);
		}
	}
	
	function animNext(sObj)
	{
		
		var n = sObj.arr.length;
		var o, item;

		//через замыкание делаем каллбэк на окончание анимации
		var obj = {'cAnim':0, 'sObj':sObj};
		var course = function(obj){
			return function() {
				obj.cAnim++;
				if (obj.cAnim == obj.sObj.arr.length) {
					sObj.animated = false;
					
					if (sObj.onEndAnimation)
						sObj.onEndAnimation.call();
					
					sObj.next();
				}
			}
		}
		var f = course(obj);

		if (sObj.onBeginAnimation)
			sObj.onBeginAnimation.call();
		
		sObj.animated = true;
		for (var i=n-1; i>=0; i--) {
			
			item = $('#sl_img'+i , sObj.self);
			o = parseInt( item.css('left'), 10 );
			o -= (sObj.imgW + sObj.margin);
			item.animate({'left':o+'px'}, 500, f);
		}
	}
	
	function animPrev(sObj)
	{
		var n = sObj.arr.length;
		var o, item;
		
		//через замыкание делаем каллбэк на окончание анимации
		var obj = {'cAnim':0, 'sObj':sObj};
		var course = function(obj){
			return function() {
				obj.cAnim++;
				if (obj.cAnim == obj.sObj.arr.length) {
					sObj.animated = false;
					
					if (sObj.onEndAnimation)
						sObj.onEndAnimation.call();
					
					sObj.prev();
				}
			}
		}
		var f = course(obj);
		
		sObj.animated = true;
		if (sObj.onBeginAnimation)
			sObj.onBeginAnimation.call();
		for (var i=0; i<n; i++) {
			
			item = $('#sl_img'+i , sObj.self);
			o = parseInt( item.css('left'), 10 );
			o += sObj.imgW + sObj.margin;
			item.animate({'left':o+'px'}, 500, f);
		}
	}
	
	function SliderObject(self, params)
	{
		//клиентские обработчики событий
		this.onInited        = null;
		this.onSwappingStart = null;
		this.onSwappingEnd   = null;
		this.onLoadingError  = null;
		
		this.timeoutID       = null;
		
		//выставляем клиентские обработчики событий
		if (params.handlers) {
			
			for (var hName in params.handlers) {
				this[hName] = params.handlers[hName];
			}
		}

		this.current = 0;
		this.list    = params.images;
		
		this.imgW    = params.imgW;
		this.imgH    = params.imgH;
		this.margin  = params.margin;
		this.scrW    = params.scrW || 960;		
		
		this.animated = false;
		
		this.self    = self;
		
		this.baseOffset = Math.round((this.scrW - this.imgW) / 2) + 7//- this.margin;
		
		this.dOffset    = this.imgW + this.margin;		
		
		var n = (params.images.length > 5) ? 5 : params.images.length;
		
		this.arr     = new Array(n);
		
		for (var i = 0; i < n; i++)
			this.arr[i] = i;
		
		
		this.getOffset = function(num)
		{
			return this.baseOffset - this.dOffset * num;
		}
		
		this.arrLeftShift = function()
		{
			//первую картинку удаляем и подгружаем в нее новую
			//берем первую картину. Переименуем ее во временный id
			var item = $('#sl_img0', this.self);
			item.attr('id', 'sl_img5');
			//перемещаем ее в конец
			var o = this.baseOffset + 2*this.dOffset;
			item.css('left', o+'px');
			
			//сдвиг
			var i;
			var n = this.arr.length;
			for (i=1; i<n; i++) {
				this.arr[i-1] = this.arr[i];
				$('#sl_img'+i, this.self).attr('id', 'sl_img'+(i-1));
			}
			this.arr[n-1] = this.arr[n-1] + 1;
			
			item.attr('id', 'sl_img4');
			
			//вызываем пользовательский обработчик начала подгрузки картинки
			if (this.onSwappingStart)
				this.onSwappingStart.call();
			
			
		   
			//вешаем на новую картинку пользовательский обработчик окончания подгрузки
			item.unbind('load');
			item.bind('load', this, function(event){

				if (event.data.timeoutID != null) {
					
					alert('Clear: ' + event.data.timeoutID)
					clearTimeout(event.data.timeoutID);
					event.data.timeoutID = null;
				}
				
				if (event.data.onSwappingEnd)
					event.data.onSwappingEnd.call();
			});
	
			
			//теперь начинаем подгружать новую картинку
			
			if (this.timeoutID == null) {
				this.timeoutID = setTimeout(this.onLoadingError, 12000);
			}
			else {
				clearTimeout(this.timeoutID);
				this.timeoutID = setTimeout(this.onLoadingError, 12000);
			}
			alert('Set: ' + this.timeoutID)
			item.attr('src', this.list[this.arr[n-1]]);
			
			//alert(this.list[this.arr[n-1]]);
			//alert($(this.self).html());
		}
		
		this.arrRightShift = function()
		{
			//посл картинку удаляем и подгружаем в нее новую
			//берем посл картинку. Переименуем ее во временный id
			var item = $('#sl_img4', this.self);
			item.attr('id', 'sl_img5');
			//перемещаем ее в начало
			var o = this.baseOffset-2*this.dOffset;
			item.css('left', o+'px');
			
			//сдвиг
			for (var i = this.arr.length-1; i>0; i--) {
				
				$('#sl_img'+(i-1), this.self).attr('id', 'sl_img'+i);
				this.arr[i] = this.arr[i-1];
			}
			this.arr[0] = this.arr[0]-1;
			item.attr('id', 'sl_img0');
				
			//вызываем пользовательский обработчик начала подгрузки картинки
			if (this.onSwappingStart)
				this.onSwappingStart.call();
			
			//вешаем на новую картинку пользовательский обработчик окончания подгрузки				
			$('img', this.self).eq(0).bind('load', this, function(event){

				
				
				if (event.data.timeoutID != null) {
					clearTimeout(event.data.timeoutID);
					event.data.timeoutID = null;
					alert('Clear: ' + event.data.timeoutID)
				}
				
				if (event.data.onSwappingEnd)
					event.data.onSwappingEnd.call();
			});
		
			//теперь начинаем подгружать новую картинку
			
			if (this.timeoutID == null) {
				this.timeoutID = setTimeout(this.onLoadingError, 12000);
			}
			else {
				clearTimeout(this.timeoutID);
				this.timeoutID = setTimeout(this.onLoadingError, 12000);
			}
				
				
			alert('Set: ' + this.timeoutID)
			item.attr('src', this.list[this.arr[0]]);
			//alert($(this.self).html());
		}
		
		this.next = function() 
		{
			//alert ('obj next');
			this.current++;
			if (this.current == 3) {
				
				//есть ли у текущего посл элемента сосед справа в общем списке
				if (this.arr[ this.arr.length-1 ] < this.list.length-1) {
					this.arrLeftShift();
					this.current = 2;
				}
			}
			
			
		}
		
		this.prev = function()
		{
			//alert ('obj prev');
			this.current--;
			if (this.current == 1) {
				
				//есть ли у текущего первого элемента сосед слева в общем списке
				if (this.arr[0] > 0) {
					this.arrRightShift();
					this.current = 2;
				}
			}
		}
	}
	
	
})(jQuery)